/* global style definitions */


body {
    width: 100%;
    max-width: 70em;
    min-width: 50em;
    margin: 0 auto 1em auto; 
    background: #FFF;
    font: normal 100% sans-serif;
}

img {
    border: 0;
}

nav a { 
    text-decoration: none;
}

nav a:hover {
    text-decoration: underline;
}

nav ul { /* all lists */
    padding: 0;
    margin: 0;
    list-style: none;
    }

nav li a{
    display: block;
    }

/*Site Navigation*/

nav#site {
    vertical-align: middle;
    background: #369;
    float: left;
    height: 2.375em;
    width: 100%;
}

nav#site a{
    display: block;
    float: left;
    
}

nav#site ul li,
nav#user ul li {
    display: inline;
}

nav#site li a{
    font-size: 80%;
    margin-right: 0.5em;
    text-transform: lowercase;
    color: #FFF;
    }

nav#site ul{
/*    vertical-align: middle;
    margin-top: 0.1em;*/
    margin: 0.5em 0;
    height: 100%;
    float: right;
}

nav#site form{
    display: block;
    float: left;
}

nav#site form input{
    border: 0;
}

/*User navigation*/

nav#user {
    width: 100%;
    height: 1.25em;
    float: left;
    clear: left;
    background: #59C;
    border-top: #CCC 1px solid;
}

nav#user .username {
    float: left;
    font-weight: bold;
    font-size: 90%;
}

nav#user ul {
    float: right;
}

nav#user a {
}

nav#user a {
    color: #FFF;
    float: left;
    margin-right: 0.5em;
    font-weight: bold;
    font-size: 90%;
}

/*actions and object navigation*/

nav#actions {
    margin-top: 0.1em;
    background: #AAA;
    clear: left;
    float: left;
    width: 20%;
}

nav#actions a{
    color: #FFF;
    font-weight: bold;
}

nav#actions ul{
    border-top: 0.1em solid #FFF;
    padding-left: 0.1em;
}

nav#actions li a{
    display: inline;
}

#Desktop {
    background: #FFF;
    float: left;
    width: 78%;
    margin: 1em 0  0 1em;
}

#Desktop a {
    text-decoration: none;
    border-bottom: 1px dotted #000;
}