/*Styles for legacy templates*/

table.FormLayout {
    width: 70%;
}

tr {
    vertical-align: text-bottom;
}

textarea {
    width: 100%;
}

input[type="text"] {
    width:100%;
}

#DesktopStatusBar {
    background: #EEE;
}