/*Action and content icons*/

.icon {
    padding-left: 1.5em;
}

/*content icons*/

.Folder {
    background: url(folder_icon.gif) no-repeat 0.1em;
}

.Document {
    background: url(document_icon.gif) no-repeat 0.1em;
}

.Image {
    background: url(image_icon.gif) no-repeat 0.1em;
}

.File {
    background: url(file_icon.gif) no-repeat 0.1em;
}

.Link {
    background: url(link_icon.gif) no-repeat 0.1em;
}

.Favorite {
    background: url(link_icon.gif) no-repeat 0.1em;
}

.News {
    background: url(newsitem_icon.gif) no-repeat 0.1em;
}

/*object icons*/

.interfaces {
    background: url(interfaces_icon.png) no-repeat 0.1em;
}

.edit {
    background: url(edit_icon.png) no-repeat 0.1em;
}

.view {
    background: url(preview_icon.png) no-repeat 0.1em;
}

.metadata {
    background: url(metadata_icon.png) no-repeat 0.1em;
}

.roles {
    background: url(preview_icon.png) no-repeat 0.1em;
}

.reply {
    background: url(reply_icon.png) no-repeat 0.1em;
}

.syndication {
    background: url(syndication_icon.png) no-repeat 0.1em;
}

/*folder icons*/

.folderContents {
    background: url(folder_icon.gif) no-repeat 0.1em;
}

.localroles {
    background: url(localroles_icon.png) no-repeat 0.1em;
}

/*workflow icons*/

.submit {
    background: url(submit_icon.png) no-repeat 0.1em;
}

.publish {
    background: url(approve_icon.png) no-repeat 0.1em;
}

.retract {
    background: url(retract_icon.png) no-repeat 0.1em;
}

.reject {
    background: url(reject_icon.png) no-repeat 0.1em;
}

.expire {
    background: url(expire_icon.png) no-repeat 0.1em;
}

.worklist {
    background: url(worklist_icon.png) no-repeat 0.1em;
}

/*user icons*/

.join {
    background: url(join_icon.png) no-repeat 0.1em;
}

.preferences {
    background: url(preferences_icon.png) no-repeat 0.1em;
}

.login {
    background: url(login_icon.png) no-repeat 0.1em;
}

.logout {
    background: url(logout_icon.png) no-repeat 0.1em;
}

.addFavorite {
    background: url(addfavorite_icon.png) no-repeat 0.1em;
}

.mystuff {
    background: url(user_icon.png) no-repeat 0.1em;
}

/*Global*/

.configPortal {
    background: url(tool_icon.png) no-repeat 0.1em;
}

.undo {
    background: url(undo_icon.png) no-repeat 0.1em;
}

.manage_members {
    background: url(members_icon.png) no-repeat 0.1em;
}